# step 1: create dummy model with pytorch
import torch

model = torch.nn.Transformer(d_model=64, nhead=4, num_encoder_layers=2, num_decoder_layers=2, batch_first=True)
model.eval()
src = torch.rand((1, 10, 64))
tgt = torch.rand((1, 20, 64))
out = model(src, tgt)
assert out.shape == (1, 20, 64)

filename = 'test_transformer.onnx'
torch.onnx.export(
        model,
        (src, tgt),
        filename,
        input_names=['src', 'tgt'],
        dynamo=True,  # see pytorch github issue #110255
        verbose=True,
        )
print('Exported a dummy transformer at', filename)

# step 2: load with onnxruntime and do the inference
import numpy as np
import onnxruntime as rt

print('ORT: available providers:', rt.get_available_providers())
session = rt.InferenceSession('test_transformer.onnx', providers=rt.get_available_providers())
print('ORT: loaded the dummy transformer')
inputs_onnx = {'src': np.random.randn(1, 10, 64).astype(np.float32),
               'tgt': np.random.randn(1, 20, 64).astype(np.float32)}
pred_onnx = session.run(None, inputs_onnx)[0]
print('pred_onnx:', pred_onnx, 'shape:', pred_onnx.shape)

print('Test OK')



#!/usr/bin/python3

import onnxruntime as rt
import numpy as np

# step 2: load with onnxruntime and do the inference
print('ORT: available providers:', rt.get_available_providers())
session = rt.InferenceSession('test_model.onnx', providers=rt.get_available_providers())
print('ORT: loaded the dummy network')
input_name = session.get_inputs()[0].name
inputs_onnx = {'feature': np.random.randn(1, 128).astype(np.float32)}
pred_onnx = session.run(None, inputs_onnx)[0]
print('pred_onnx:', pred_onnx, 'shape:', pred_onnx.shape)
assert pred_onnx.shape == (1, 1)

print('Test OK')



#!/usr/bin/python3

import numpy as np
import onnx
from onnx import helper, TensorProto

# step 1: create a dummy neural network with NumPy + ONNX
# Parameters
B = 128
N = 4096

# Random weights for a 2-layer MLP
W1 = np.random.randn(N, N).astype(np.float32)
b1 = np.random.randn(N).astype(np.float32)
W2 = np.random.randn(N, N).astype(np.float32)
b2 = np.random.randn(N).astype(np.float32)

# Create ONNX tensors for initializers
init_W1 = helper.make_tensor('W1', TensorProto.FLOAT, W1.shape, W1.flatten())
init_b1 = helper.make_tensor('b1', TensorProto.FLOAT, b1.shape, b1.flatten())
init_W2 = helper.make_tensor('W2', TensorProto.FLOAT, W2.shape, W2.flatten())
init_b2 = helper.make_tensor('b2', TensorProto.FLOAT, b2.shape, b2.flatten())

# Input and output
input_tensor = helper.make_tensor_value_info('feature', TensorProto.FLOAT, [None, N])
output_tensor = helper.make_tensor_value_info('output', TensorProto.FLOAT, [None, N])

# Nodes
node1 = helper.make_node('Gemm', inputs=['feature', 'W1', 'b1'], outputs=['hidden_linear'])
node2 = helper.make_node('Relu', inputs=['hidden_linear'], outputs=['hidden_relu'])
node3 = helper.make_node('Gemm', inputs=['hidden_relu', 'W2', 'b2'], outputs=['output'])

# Build graph
graph = helper.make_graph(
    nodes=[node1, node2, node3],
    name='DummyLargeMLP',
    inputs=[input_tensor],
    outputs=[output_tensor],
    initializer=[init_W1, init_b1, init_W2, init_b2]
)

# Build model
model = helper.make_model(graph, producer_name='onnx_large_dummy')

filename = 'test_provider.onnx'
print('[..] Exporting dummy neural network for mainly GEMM test')
onnx.save(model, filename)
print('[OK] Exported dummy neural network for mainly GEMM test:', filename)



#!/usr/bin/python3

import numpy as np
import onnxruntime as rt
import time

# step 1: create a dummy neural network with NumPy + ONNX
# Parameters
B = 128
N = 4096

# step 2: load with onnxruntime and do the inference, try different providers.
elapsed_time = {}
repeat = 10

print('ORT: available providers:', rt.get_available_providers())
session = rt.InferenceSession('test_provider.onnx', providers=rt.get_available_providers())

for provider in rt.get_available_providers():
    print('Testing provider:', provider)
    session.set_providers([provider])
    input_name = session.get_inputs()[0].name
    inputs_onnx = {input_name: np.random.randn(B, N).astype(np.float32)}
    tmp = []
    for _ in range(repeat):
        time_start = time.time()
        pred_onnx = session.run(None, inputs_onnx)[0]
        time_end = time.time()
        elapsed = time_end - time_start
        print('         elapsed:', elapsed)
        tmp.append(elapsed)
    assert pred_onnx.shape == (B, N)
    elapsed_time[provider] = sum(tmp) / float(repeat)

print(elapsed_time)
